package org.geoff.geo;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.BasicDBObjectBuilder;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.Mongo;

import org.json.simple.*;

import org.bson.types.ObjectId;




/**
 * Servlet implementation class GameMap
 */
public class GameMap extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public GameMap() {
        super();
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// TODO Auto-generated method stub
		
		// servlet request parameters: 
		// Integer x = player's current x
		// Integer y = player's current y
		// String id = player's id
		// String srchtyp = search type.  values are 'box' or 'rad'
		// Integer span = search perimeter
		
		Boolean requestIsValid = true;
		HashMap<Integer, String> errors = new HashMap<Integer, String>();
		HashMap<String,String> players = new HashMap<String,String>();
		Integer errorCount = 0;
		
		String msg = "";
		String findSpec = "";
		Integer playerCount = 0;
		String json = "";	
		
		String searchType = "rad";
		Integer searchSpan = 100;
		String playerId = "";
		double xPos = 0;
		double yPos = 0;

		if (request.getParameter("id") != null){
			playerId = request.getParameter("id");
		}
		if (request.getParameter("srchtyp") != null){
			searchType = request.getParameter("srchtyp");
		} else {
			searchType = "rad";
		}
		if (request.getParameter("span") != null){
			searchSpan = Integer.parseInt(request.getParameter("span"));
		} else {
			searchSpan = 100;
		}
		if (request.getParameter("x") != null){
			xPos = Double.parseDouble(request.getParameter("x"));
		} else {
			requestIsValid = false;
			errorCount++;
			errors.put(errorCount, "missing parameter: x coordinate");
		}
		if (request.getParameter("y") != null){
			yPos = Double.parseDouble(request.getParameter("y"));
		} else {
			requestIsValid = false;
			errorCount++;
			errors.put(errorCount, "missing parameter: y coordinate");
		}
	
		
		if (requestIsValid){
			
			ServletContext ctx = this.getServletContext();
			
			String dbHost = ctx.getInitParameter("dbhost");
			String dbName = ctx.getInitParameter("dbName");
			String historyCollection = ctx.getInitParameter("historyCollection");
			String currentLocationCollection = ctx.getInitParameter("currentLocationCollection");
			
			Mongo m = new Mongo(dbHost);
			DB db = m.getDB(dbName);
			DBCollection collLocationHistory = db.getCollection(historyCollection);
			DBCollection collCurrentLocation = db.getCollection(currentLocationCollection);
		
			BasicDBObject pos = new BasicDBObject();
			pos.put("x", xPos);
			pos.put("y", yPos);
			Date ts = new Date();
			BasicDBObject playerLocation = (BasicDBObject) BasicDBObjectBuilder.start().add("name", playerId).
			   add("ts", ts).add("loc", pos).get();

			createHistoryRecord(collLocationHistory, playerLocation);
			
			setCurrentLocation(collCurrentLocation, playerLocation, playerId);
			
			players = findPeers(collCurrentLocation, xPos, yPos, searchType, searchSpan, playerId);
		}
		
		if (errors.size() > 0){
			for ( Iterator<String> it = errors.values().iterator(); it.hasNext(); ){
				msg += (String) it.next() + ",";
			}
			
			if (msg.length() > 0){
				int m = msg.lastIndexOf(",");
				if (m > 0){
					msg = msg.substring(0, m);
				}
			}
			
		}
		
		HashMap<String,String> output = new HashMap<String,String>();
		output.put("timestamp", String.valueOf(System.currentTimeMillis()));
		output.put("errorMessage", msg); 
		output.put("playerCount", String.valueOf(players.size()));
		
		if (players != null){
			output.put("playerlist", JSONValue.toJSONString(players));
		}	
		//} else {
		//	output.put("playerlist", "{}");
		//}
		json = JSONValue.toJSONString(output);
		PrintWriter pw = response.getWriter();
		pw.println(json);
		
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// TODO Auto-generated method stub
	}
	
	/**
	 * 
	 */
	private Boolean createHistoryRecord(DBCollection coll, BasicDBObject dbo){
		// TODO check for error and handle if one occurs
		Boolean res = true;
		coll.save(dbo);
		return res;
	}

	/**
	 * 
	 */
	private Boolean setCurrentLocation(DBCollection coll, 
			BasicDBObject dbo, 
			String playerId){
		
		// TODO check for error and handle if one occurs
		Boolean res = true;
		BasicDBObject qry = new BasicDBObject("name", playerId);
		BasicDBObject doc = (BasicDBObject) coll.findOne(qry);
		ObjectId docId;
		if (doc != null){
			docId = (ObjectId) doc.get("_id");
		} else {
			docId = new ObjectId();
		}
		
		BasicDBObject updQry = new BasicDBObject("_id", docId);
		coll.update(updQry, dbo, true, false);
		
		return res;
	}
	
	/**
	 * 
	 */
	private HashMap<String,String> findPeers(DBCollection coll,  
			double xPos, 
			double yPos, 
			String searchType, 
			Integer span,
			String playerId){
		
		HashMap<String,String> peers = new HashMap<String,String>();
		BasicDBObject loc = new BasicDBObject();
		BasicDBObject within = new BasicDBObject();

		if (searchType.equalsIgnoreCase("box")){
			BasicDBList box = new BasicDBList();
			BasicDBList upperPt = new BasicDBList();
			BasicDBList lowerPt = new BasicDBList();
			
			upperPt.put(0, xPos + span);
			upperPt.put(1, yPos + span);
			lowerPt.put(0, xPos - span);
			lowerPt.put(1, yPos - span);
			
			box.put(0, lowerPt);
			box.put(1, upperPt);
			
			within.put("$box", box);
			loc.put("$within", within);

		} else {
			BasicDBList near = new BasicDBList();
			BasicDBObject center = new BasicDBObject();
			
			near.put(0, xPos);
			near.put(1, yPos);
			loc.put("$near", near);
			loc.put("$maxDistance", span);
		}
				
		BasicDBObject query = new BasicDBObject();
		query.put("loc", loc);
		
		DBCursor cur = coll.find(query);
		
		while(cur.hasNext()){
			BasicDBObject playerLoc = (BasicDBObject) cur.next();
			String playerName = (String) playerLoc.get("name");
			if (!playerName.equalsIgnoreCase(playerId)){
				BasicDBObject playerPt = (BasicDBObject) playerLoc.get("loc");
				String playerXCoord = String.valueOf(playerPt.get("x"));
				String playerYCoord = String.valueOf(playerPt.get("y"));
				peers.put(playerName, playerXCoord + "," + playerYCoord);
			}
		}
		
		return peers;
	}
	
	/**
	 * 
	 */
	private HashMap<String,String> findPeersInBox(){
		HashMap<String,String> peers = new HashMap<String,String>();
		

		return peers;
	}
	
	/**
	 * 
	 */
	private HashMap<String,String> findPeersWithinRadius(){
		HashMap<String,String> peers = new HashMap<String,String>();
		

		return peers;
	}
	
	

}
